package com.qmth.themis.backend.api;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.themis.business.base.BasePage;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TEExamCourse;
import com.qmth.themis.business.enums.FieldUniqueEnum;
import com.qmth.themis.business.service.TEExamCourseService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 考试科目 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "考试科目Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/exam")
public class TEExamCourseController {

    @Resource
    TEExamCourseService teExamCourseService;

    @ApiOperation(value = "考试科目修改/新增接口")
    @RequestMapping(value = "/course/save", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result save(@ApiParam(value = "考试科目信息", required = true) @RequestBody TEExamCourse teExamCourse) {
        if (Objects.isNull(teExamCourse) || Objects.equals(teExamCourse, "")) {
            throw new BusinessException(ExceptionResultEnum.COURSE_INFO_IS_NULL);
        }
        try {
            if (Objects.isNull(teExamCourse.getId())) {
                teExamCourse.setId(Constants.idGen.next());
            }
            teExamCourseService.saveOrUpdate(teExamCourse);
        } catch (Exception e) {
            e.printStackTrace();
            if (e instanceof DuplicateKeyException) {
                String errorColumn = e.getCause().toString();
                String columnStr = errorColumn.substring(errorColumn.lastIndexOf("key") + 3, errorColumn.length()).replaceAll("'", "");
                throw new BusinessException("机构id[" + teExamCourse.getExamId() + "]下的" + FieldUniqueEnum.convertToCode(columnStr) + "数据不允许重复插入");
            } else if (e instanceof BusinessException) {
                throw new BusinessException(e.getMessage());
            } else {
                throw new RuntimeException(e);
            }
        }
        teExamCourseService.updateExamCourseCacheBean(teExamCourse.getExamId(), teExamCourse.getCourseCode());
        return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.SUCCESS));
    }

    @ApiOperation(value = "考试科目查询接口")
    @RequestMapping(value = "/course/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考试科目信息", response = TEExamCourse.class)})
    public Result query(@ApiParam(value = "考试批次id", required = true) @RequestParam Long examId, @ApiParam(value = "科目编码", required = false) @RequestParam(required = false) String courseCode, @ApiParam(value = "科目名称", required = false) @RequestParam(required = false) String courseName, @ApiParam(value = "是否已绑定试卷", required = false) @RequestParam(required = false) Integer hasPaper, @ApiParam(value = "分页页码", required = true) @RequestParam int pageNumber, @ApiParam(value = "分页数", required = true) @RequestParam int pageSize) {
        if (Objects.isNull(examId) || Objects.equals(examId, "")) {
            throw new BusinessException(ExceptionResultEnum.EXAM_ID_IS_NULL);
        }
        IPage<TEExamCourse> examCourseIPage = teExamCourseService.examCourseQuery(new Page<>(pageNumber, pageSize), examId, courseCode, courseName, hasPaper);
        BasePage basePage = new BasePage(examCourseIPage.getRecords(), examCourseIPage.getCurrent(), examCourseIPage.getSize(), examCourseIPage.getTotal());
        Map map = new HashMap<>();
        map.put(SystemConstant.RECORDS, basePage);
        return ResultUtil.ok(map);
    }
}
