package com.qmth.themis.backend.api;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.gson.Gson;
import com.qmth.themis.business.annotation.ApiJsonObject;
import com.qmth.themis.business.annotation.ApiJsonProperty;
import com.qmth.themis.business.base.BasePage;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.cache.TEStudentCacheDto;
import com.qmth.themis.business.dto.response.TEStudentDto;
import com.qmth.themis.business.dto.response.TEStudentExamRecordDto;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.entity.TEStudent;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.service.TEStudentService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.util.ServletUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 学生档案 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "学生档案Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/student")
public class TEStudentController {

    @Resource
    TEStudentService teStudentService;

    @Resource
    RedisUtil redisUtil;

    @ApiOperation(value = "学生查询接口")
    @RequestMapping(value = "/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "考生信息", response = TEStudentDto.class)})
    public Result query(@ApiParam(value = "证件号", required = false) @RequestParam(required = false) String identity, @ApiParam(value = "姓名", required = false) @RequestParam(required = false) String name, @ApiParam(value = "是否启用", required = false) @RequestParam(required = false) Integer enable, @ApiParam(value = "分页页码", required = true) @RequestParam int pageNumber, @ApiParam(value = "分页数", required = true) @RequestParam int pageSize) {
        IPage<TEStudentDto> teExamStudentIPage = teStudentService.studentQuery(new Page<>(pageNumber, pageSize), identity, name, enable);
        BasePage basePage = new BasePage(teExamStudentIPage.getRecords(), teExamStudentIPage.getCurrent(), teExamStudentIPage.getSize(), teExamStudentIPage.getTotal());
        Map map = new HashMap<>();
        map.put(SystemConstant.RECORDS, basePage);
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "学生停用/启用接口")
    @RequestMapping(value = "/toggle", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    @Transactional
    public Result studentToggle(@ApiJsonObject(name = "studentToggle", value = {
            @ApiJsonProperty(key = "id", type = "long", example = "1", description = "学生ID"),
            @ApiJsonProperty(key = "enable", type = "long", example = "1", description = "是否启用")
    }) @ApiParam(value = "学生信息", required = true) @RequestBody Map<String, Object> mapParameter) {
        if (Objects.isNull(mapParameter.get("id")) || Objects.equals(mapParameter.get("id"), "")) {
            throw new BusinessException(ExceptionResultEnum.STUDENT_ID_IS_NULL);
        }
        Long id = Long.parseLong(String.valueOf(mapParameter.get("id")));
        if (Objects.isNull(mapParameter.get("enable")) || Objects.equals(mapParameter.get("enable"), "")) {
            throw new BusinessException(ExceptionResultEnum.ENABLE_IS_NULL);
        }
        Integer enable = Integer.parseInt(String.valueOf(mapParameter.get("enable")));
        TEStudent teStudent = teStudentService.getById(id);
        if (Objects.isNull(teStudent)) {
            throw new BusinessException(ExceptionResultEnum.STUDENT_NO);
        }
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
        teStudent.setEnable(enable);
        teStudent.setUpdateId(tbUser.getId());
        teStudentService.updateById(teStudent);
        Gson gson = new Gson();
        TEStudentCacheDto teStudentCacheDto = gson.fromJson(gson.toJson(teStudent), TEStudentCacheDto.class);
        redisUtil.setStudent(teStudent.getId(), teStudentCacheDto);
        return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.SUCCESS));
    }

    @ApiOperation(value = "学生修改密码接口")
    @RequestMapping(value = "/updatePwd", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    @Transactional
    public Result studentUpdatePwd(@ApiJsonObject(name = "studentUpdatePwd", value = {
            @ApiJsonProperty(key = "id", type = "long", example = "1", description = "用户ID"),
            @ApiJsonProperty(key = "password", description = "新密码")
    }) @ApiParam(value = "学生信息", required = true) @RequestBody Map<String, Object> mapParameter) {
        if (Objects.isNull(mapParameter.get("id")) || Objects.equals(mapParameter.get("id"), "")) {
            throw new BusinessException(ExceptionResultEnum.STUDENT_ID_IS_NULL);
        }
        Long id = Long.parseLong(String.valueOf(mapParameter.get("id")));
        if (Objects.isNull(mapParameter.get("password")) || Objects.equals(mapParameter.get("password"), "")) {
            throw new BusinessException(ExceptionResultEnum.PASSWORD_IS_NULL);
        }
        String password = String.valueOf(mapParameter.get("password"));
        TEStudent teStudent = teStudentService.getById(id);
        if (Objects.isNull(teStudent)) {
            throw new BusinessException(ExceptionResultEnum.STUDENT_INFO_IS_NULL);
        }
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
        teStudent.setPassword(password);
        teStudent.setUpdateId(tbUser.getId());
        teStudentService.updateById(teStudent);
        Gson gson = new Gson();
        TEStudentCacheDto teStudentCacheDto = gson.fromJson(gson.toJson(teStudent), TEStudentCacheDto.class);
        redisUtil.setStudent(teStudent.getId(), teStudentCacheDto);
        return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.SUCCESS));
    }

    @ApiOperation(value = "学生考试记录查询接口")
    @RequestMapping(value = "/exam_record/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "学生考试记录信息", response = TOeExamRecord.class)})
    public Result examRecordQuery(@ApiParam(value = "学生ID", required = true) @RequestParam Long studentId, @ApiParam(value = "考试id", required = false) @RequestParam(required = false) Long examId, @ApiParam(value = "分页页码", required = true) @RequestParam int pageNumber, @ApiParam(value = "分页数", required = true) @RequestParam int pageSize) {
        if (Objects.isNull(studentId) || Objects.equals(studentId, "")) {
            throw new BusinessException(ExceptionResultEnum.STUDENT_ID_IS_NULL);
        }
        IPage<TEStudentExamRecordDto> teStudentExamRecordDtoIPage = teStudentService.studentExamRecordQuery(new Page<>(pageNumber, pageSize), studentId, examId);
        BasePage basePage = new BasePage(teStudentExamRecordDtoIPage.getRecords(), teStudentExamRecordDtoIPage.getCurrent(), teStudentExamRecordDtoIPage.getSize(), teStudentExamRecordDtoIPage.getTotal());
        Map map = new HashMap<>();
        map.put(SystemConstant.RECORDS, basePage);
        return ResultUtil.ok(map);
    }
}
