package com.qmth.themis.business.dto.response;

import com.qmth.themis.business.cache.bean.ExamActivityCacheBean;
import com.qmth.themis.business.cache.bean.ExamCacheBean;
import com.qmth.themis.business.cache.bean.ExamCourseCacheBean;
import com.qmth.themis.business.cache.bean.ExamStudentCacheBean;
import com.qmth.themis.business.enums.EntryAuthenticationPolicyEnum;
import com.qmth.themis.business.enums.HardwareTestEnum;
import com.qmth.themis.business.enums.MonitorVideoSourceEnum;

import java.io.Serializable;
import java.util.*;

/**
 * @Description: 考试场次dto
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/8/3
 */
public class TEExamActivityDto implements Serializable {

    private Long id;//考试id
    private String code;//考试场次代码
    private Long examStudentId;//考生id
    private String courseCode;//科目代码
    private String courseName;//科目名称
    private Long prepareTime;
    private Long minStartTime;
    private Long maxStartTime;
    private Long maxFinishTime;
    private String monitorVideoSourceStr;
    private List<String> monitorVideoSource;
    private Integer openingSeconds;//允许开考开放时长(分钟)，相当于迟到时间
    private Integer activityOpeningSeconds;//允许开考开放时长(分钟)，相当于迟到时间
    private Integer prepareSeconds;//提前多长时间开始候考(分钟)
    private Integer activityPrepareSeconds;//提前多长时间开始候考(分钟)
    private Integer minDurationSeconds;//最短考试时长，相当于考试冻结时间(分钟)
    private Integer maxDurationSeconds;//最大考试时长
    private Integer activityMaxDurationSeconds;//最大考试时长
    private Integer examCount;//允许考试次数
    private Integer forceFinish;//是否在结束时间集中强制收卷，0：不强制，1：强制
    private String entryAuthenticationPolicy;//开考身份验证策略，off：关闭，face_verify_optional：非强制人脸验证，face_verify_force：强制人脸验证，liveness：活体验证
    private Integer inProcessFaceVerify;//考试过程中人脸检测是否开启，0：不开启，1：开启
    private Integer inProcessFaceStrangerIgnore;//考试过程中人脸检测是否忽略陌生人 ，0：不忽略，1：忽略
    private Integer inProcessLivenessVerify;//考试过程中是否启用活体检测 ，0：不启用，1：启用
    private String inProcessLivenessFixedRangeStr;
    private List<Integer> inProcessLivenessFixedRange;//考试过程中活体检测间隔时间
    private String inProcessLivenessJudgePolicy;//考试过程中活体检测结果判定规则，any：任意一次通过，all：全部都要通过，more：通过次数大于失败次数
    private Integer cameraPhotoUpload;//是否允许使用摄像头拍照答题，0：不允许，1：允许
    private Date startTime;//考场开始时间
    private Date finishTime;//考场结束时间
    private Integer leftExamCount;//剩余考试次数
    private Integer mobilePhotoUpload;//是否允许使用移动端拍照答题，0：不开启，1：开启
    private Boolean monitorAudioEnable = false;//客户端监控是否启用音频与麦克风
    private Long examId;//考试id
    private List<String> hardwareTest;//硬件检测

    public TEExamActivityDto() {

    }

    public TEExamActivityDto(ExamCacheBean ec, ExamActivityCacheBean examActivityCacheBean, ExamStudentCacheBean examStudentCacheBean, Long examStudentId,ExamCourseCacheBean examCourseCacheBean) {
        this.id = examActivityCacheBean.getId();
        this.code = examActivityCacheBean.getCode();
        this.examStudentId = examStudentId;
        this.courseCode = examStudentCacheBean.getCourseCode();
        this.courseName = examStudentCacheBean.getCourseName();
        if (Objects.nonNull(examActivityCacheBean.getPrepareSeconds())) {
            this.prepareTime = examActivityCacheBean.getStartTime().getTime() - (examActivityCacheBean.getPrepareSeconds() * 1000);
        } else if (Objects.nonNull(ec.getPrepareSeconds())) {
            this.prepareTime = examActivityCacheBean.getStartTime().getTime() - (ec.getPrepareSeconds() * 1000);
        }
        if (Objects.nonNull(ec.getMinDurationSeconds())) {
            this.minStartTime = examActivityCacheBean.getStartTime().getTime() + (ec.getMinDurationSeconds() * 1000);
        }
        if (Objects.nonNull(examActivityCacheBean.getMaxDurationSeconds())) {
            this.maxStartTime = examActivityCacheBean.getStartTime().getTime() + (examActivityCacheBean.getMaxDurationSeconds() * 1000);
        } else if (Objects.nonNull(ec.getMaxDurationSeconds())) {
            this.maxStartTime = examActivityCacheBean.getStartTime().getTime() + (ec.getMaxDurationSeconds() * 1000);
        }
        if (Objects.nonNull(examActivityCacheBean.getFinishTime())) {
            this.maxFinishTime = examActivityCacheBean.getFinishTime().getTime();
        } else if (Objects.nonNull(ec.getEndTime())) {
            this.maxFinishTime = ec.getEndTime().getTime();
        }
        this.cameraPhotoUpload = ec.getCameraPhotoUpload();
        this.entryAuthenticationPolicy = ec.getEntryAuthenticationPolicy().name();
        if (Objects.nonNull(ec.getMonitorVideoSource()) && !Objects.equals(ec.getMonitorVideoSource().toString().trim().replaceAll(" ", ""), "")) {
            this.setMonitorVideoSource(Arrays.asList(ec.getMonitorVideoSource().trim().toUpperCase().split(",")));
            //加入monitorAudioEnable逻辑
            if (ec.getMonitorVideoSource().toUpperCase().contains(MonitorVideoSourceEnum.CLIENT_SCREEN.name()) || ec.getMonitorVideoSource().toUpperCase().contains(MonitorVideoSourceEnum.CLIENT_CAMERA.name())) {
                this.monitorAudioEnable = true;
            }
            //加入hardwareTest逻辑
            if (ec.getMonitorVideoSource().toUpperCase().contains(MonitorVideoSourceEnum.CLIENT_CAMERA.name()) || (Objects.nonNull(this.getEntryAuthenticationPolicy()) && (Objects.equals(this.getEntryAuthenticationPolicy(), EntryAuthenticationPolicyEnum.LIVENESS_VERIFY.name()) || Objects.equals(this.getEntryAuthenticationPolicy(), EntryAuthenticationPolicyEnum.FACE_VERIFY_FORCE.name()))) || (Objects.nonNull(this.getCameraPhotoUpload()) && this.getCameraPhotoUpload() == 1)) {
                if (Objects.isNull(this.hardwareTest)) {
                    this.hardwareTest = new ArrayList<>();
                }
                this.hardwareTest.add(HardwareTestEnum.CAMERA.name());
                //取course缓存
                if (Objects.nonNull(examCourseCacheBean) && Objects.nonNull(examCourseCacheBean.getHasAudio()) && examCourseCacheBean.getHasAudio() == 1) {
                    this.hardwareTest.add(HardwareTestEnum.AUDIOPLAY.name());
                }
            }
        } else {
            this.setMonitorVideoSource(null);
        }
        this.openingSeconds = ec.getOpeningSeconds();
        this.activityOpeningSeconds = examActivityCacheBean.getOpeningSeconds();
        this.prepareSeconds = ec.getPrepareSeconds();
        this.activityPrepareSeconds = examActivityCacheBean.getPrepareSeconds();
        this.minDurationSeconds = ec.getMinDurationSeconds();
        this.maxDurationSeconds = ec.getMaxDurationSeconds();
        this.activityMaxDurationSeconds = examActivityCacheBean.getMaxDurationSeconds();
        this.examCount = ec.getExamCount();
        this.forceFinish = ec.getForceFinish();
        this.inProcessFaceVerify = ec.getInProcessFaceVerify();
        this.inProcessFaceStrangerIgnore = ec.getInProcessFaceStrangerIgnore();
        this.inProcessLivenessVerify = ec.getInProcessLivenessVerify();
        if (Objects.nonNull(ec.getInProcessLivenessFixedRange())) {
            String[] longs = ec.getInProcessLivenessFixedRange().trim().split(",");
            List inProcessLivenessFixedRange = new ArrayList();
            for (int i = 0; i < longs.length; i++) {
                Integer integer = Integer.valueOf(longs[i].trim());
                inProcessLivenessFixedRange.add(integer);
            }
            if (Objects.equals(inProcessLivenessFixedRange.toString().trim().replaceAll(" ", ""), "")) {
                this.setInProcessLivenessFixedRange(null);
            } else {
                this.setInProcessLivenessFixedRange(inProcessLivenessFixedRange);
            }
        }
        this.inProcessLivenessJudgePolicy = ec.getInProcessLivenessJudgePolicy().name();
        this.startTime = examActivityCacheBean.getStartTime();
        this.finishTime = examActivityCacheBean.getFinishTime();
        this.leftExamCount = examStudentCacheBean.getLeftExamCount();
        this.mobilePhotoUpload = ec.getMobilePhotoUpload();
        this.examId = ec.getId();
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Boolean getMonitorAudioEnable() {
        return monitorAudioEnable;
    }

    public void setMonitorAudioEnable(Boolean monitorAudioEnable) {
        this.monitorAudioEnable = monitorAudioEnable;
    }

    public List<String> getHardwareTest() {
        return hardwareTest;
    }

    public void setHardwareTest(List<String> hardwareTest) {
        this.hardwareTest = hardwareTest;
    }

    public Integer getMobilePhotoUpload() {
        return mobilePhotoUpload;
    }

    public void setMobilePhotoUpload(Integer mobilePhotoUpload) {
        this.mobilePhotoUpload = mobilePhotoUpload;
    }

    public Integer getLeftExamCount() {
        return leftExamCount;
    }

    public void setLeftExamCount(Integer leftExamCount) {
        this.leftExamCount = leftExamCount;
    }

    public String getMonitorVideoSourceStr() {
        return monitorVideoSourceStr;
    }

    public void setMonitorVideoSourceStr(String monitorVideoSourceStr) {
        this.monitorVideoSourceStr = monitorVideoSourceStr;
    }

    public String getInProcessLivenessFixedRangeStr() {
        return inProcessLivenessFixedRangeStr;
    }

    public void setInProcessLivenessFixedRangeStr(String inProcessLivenessFixedRangeStr) {
        this.inProcessLivenessFixedRangeStr = inProcessLivenessFixedRangeStr;
    }

    public Long getPrepareTime() {
        return prepareTime;
    }

    public void setPrepareTime(Long prepareTime) {
        this.prepareTime = prepareTime;
    }

    public Long getMinStartTime() {
        return minStartTime;
    }

    public void setMinStartTime(Long minStartTime) {
        this.minStartTime = minStartTime;
    }

    public Long getMaxStartTime() {
        return maxStartTime;
    }

    public void setMaxStartTime(Long maxStartTime) {
        this.maxStartTime = maxStartTime;
    }

    public Long getMaxFinishTime() {
        return maxFinishTime;
    }

    public void setMaxFinishTime(Long maxFinishTime) {
        this.maxFinishTime = maxFinishTime;
    }

    public List<String> getMonitorVideoSource() {
        return monitorVideoSource;
    }

    public void setMonitorVideoSource(List<String> monitorVideoSource) {
        this.monitorVideoSource = monitorVideoSource;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public Long getExamStudentId() {
        return examStudentId;
    }

    public void setExamStudentId(Long examStudentId) {
        this.examStudentId = examStudentId;
    }

    public String getCourseCode() {
        return courseCode;
    }

    public void setCourseCode(String courseCode) {
        this.courseCode = courseCode;
    }

    public String getCourseName() {
        return courseName;
    }

    public void setCourseName(String courseName) {
        this.courseName = courseName;
    }

    public Integer getOpeningSeconds() {
        return openingSeconds;
    }

    public void setOpeningSeconds(Integer openingSeconds) {
        this.openingSeconds = openingSeconds;
    }

    public Integer getActivityOpeningSeconds() {
        return activityOpeningSeconds;
    }

    public void setActivityOpeningSeconds(Integer activityOpeningSeconds) {
        this.activityOpeningSeconds = activityOpeningSeconds;
    }

    public Integer getPrepareSeconds() {
        return prepareSeconds;
    }

    public void setPrepareSeconds(Integer prepareSeconds) {
        this.prepareSeconds = prepareSeconds;
    }

    public Integer getActivityPrepareSeconds() {
        return activityPrepareSeconds;
    }

    public void setActivityPrepareSeconds(Integer activityPrepareSeconds) {
        this.activityPrepareSeconds = activityPrepareSeconds;
    }

    public Integer getMinDurationSeconds() {
        return minDurationSeconds;
    }

    public void setMinDurationSeconds(Integer minDurationSeconds) {
        this.minDurationSeconds = minDurationSeconds;
    }

    public Integer getMaxDurationSeconds() {
        return maxDurationSeconds;
    }

    public void setMaxDurationSeconds(Integer maxDurationSeconds) {
        this.maxDurationSeconds = maxDurationSeconds;
    }

    public Integer getActivityMaxDurationSeconds() {
        return activityMaxDurationSeconds;
    }

    public void setActivityMaxDurationSeconds(Integer activityMaxDurationSeconds) {
        this.activityMaxDurationSeconds = activityMaxDurationSeconds;
    }

    public void setInProcessLivenessFixedRange(List<Integer> inProcessLivenessFixedRange) {
        this.inProcessLivenessFixedRange = inProcessLivenessFixedRange;
    }

    public Integer getExamCount() {
        return examCount;
    }

    public void setExamCount(Integer examCount) {
        this.examCount = examCount;
    }

    public Integer getForceFinish() {
        return forceFinish;
    }

    public void setForceFinish(Integer forceFinish) {
        this.forceFinish = forceFinish;
    }

    public String getEntryAuthenticationPolicy() {
        return entryAuthenticationPolicy;
    }

    public void setEntryAuthenticationPolicy(String entryAuthenticationPolicy) {
        this.entryAuthenticationPolicy = entryAuthenticationPolicy;
    }

    public Integer getInProcessFaceVerify() {
        return inProcessFaceVerify;
    }

    public void setInProcessFaceVerify(Integer inProcessFaceVerify) {
        this.inProcessFaceVerify = inProcessFaceVerify;
    }

    public Integer getInProcessFaceStrangerIgnore() {
        return inProcessFaceStrangerIgnore;
    }

    public void setInProcessFaceStrangerIgnore(Integer inProcessFaceStrangerIgnore) {
        this.inProcessFaceStrangerIgnore = inProcessFaceStrangerIgnore;
    }

    public Integer getInProcessLivenessVerify() {
        return inProcessLivenessVerify;
    }

    public void setInProcessLivenessVerify(Integer inProcessLivenessVerify) {
        this.inProcessLivenessVerify = inProcessLivenessVerify;
    }

    public List<Integer> getInProcessLivenessFixedRange() {
        return inProcessLivenessFixedRange;
    }

    public String getInProcessLivenessJudgePolicy() {
        return inProcessLivenessJudgePolicy;
    }

    public void setInProcessLivenessJudgePolicy(String inProcessLivenessJudgePolicy) {
        this.inProcessLivenessJudgePolicy = inProcessLivenessJudgePolicy;
    }

    public Integer getCameraPhotoUpload() {
        return cameraPhotoUpload;
    }

    public void setCameraPhotoUpload(Integer cameraPhotoUpload) {
        this.cameraPhotoUpload = cameraPhotoUpload;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getFinishTime() {
        return finishTime;
    }

    public void setFinishTime(Date finishTime) {
        this.finishTime = finishTime;
    }
}
