package com.qmth.themis.business.service.impl;

import java.io.File;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.base.BasePage;
import com.qmth.themis.business.bean.backend.InvigilateListBean;
import com.qmth.themis.business.bean.backend.InvigilateListQueryBean;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.cache.bean.ExamCacheBean;
import com.qmth.themis.business.cache.bean.ExamStudentAnswerCacheBean;
import com.qmth.themis.business.cache.bean.ExamStudentCacheBean;
import com.qmth.themis.business.cache.bean.ExamStudentPaperStructCacheBean;
import com.qmth.themis.business.cache.bean.ObjectiveAnswerCacheBean;
import com.qmth.themis.business.config.SystemConfig;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dao.TOeExamRecordMapper;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.dto.response.TEExamUnFinishDto;
import com.qmth.themis.business.entity.TEExamStudent;
import com.qmth.themis.business.entity.TOeExamAnswer;
import com.qmth.themis.business.entity.TOeExamRecord;
import com.qmth.themis.business.enums.ExamRecordStatusEnum;
import com.qmth.themis.business.enums.ExamTypeEnum;
import com.qmth.themis.business.enums.LivenessTypeEnum;
import com.qmth.themis.business.enums.MqTagEnum;
import com.qmth.themis.business.enums.MqTopicEnum;
import com.qmth.themis.business.enums.ObjectiveScorePolicyEnum;
import com.qmth.themis.business.enums.VerifyExceptionEnum;
import com.qmth.themis.business.service.MqDtoService;
import com.qmth.themis.business.service.TEExamPaperService;
import com.qmth.themis.business.service.TEExamService;
import com.qmth.themis.business.service.TEExamStudentService;
import com.qmth.themis.business.service.TOeExamAnswerService;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.OssUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.util.FileUtil;
import com.qmth.themis.common.util.SimpleBeanUtil;

/**
 * @Description: 考试记录 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TOeExamRecordServiceImpl extends ServiceImpl<TOeExamRecordMapper, TOeExamRecord> implements TOeExamRecordService {
	private final static Logger log = LoggerFactory.getLogger(TOeExamRecordServiceImpl.class);
	
	private SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd");
	
    @Resource
    MqDtoService mqDtoService;
	
    @Resource
    TOeExamRecordMapper tOeExamRecordMapper;

    @Resource
    RedisUtil redisUtil;

    @Resource
    TEExamPaperService examPaperService;

    @Resource
    TEExamService examService;

    @Resource
    TEExamService teExamService;
    
    @Resource
    SystemConfig systemConfig;
    
    @Resource
    TOeExamAnswerService examAnswerService;
    
    @Resource
    TEExamStudentService examStudentService;

    /**
     * 获取考试未完列表
     *
     * @param studentId
     * @param examId
     * @param orgId
     * @return
     */
    @Override
    public Map getUnFinishExam(Long studentId, Long examId, Long orgId) {
        TEExamUnFinishDto teExamUnFinishDto = tOeExamRecordMapper.getUnFinishExam(studentId, examId, orgId);
        if (Objects.nonNull(teExamUnFinishDto)) {
            Map finalMap = new HashMap();
            Map<String, Object> waitingMap = new HashMap();
            waitingMap.put("id", teExamUnFinishDto.getId());
            waitingMap.put("examActivityId", teExamUnFinishDto.getExamActivityId());
            waitingMap.put("name", teExamUnFinishDto.getName());
            waitingMap.put("mode", teExamUnFinishDto.getMode());
            waitingMap.put("preNotice", teExamUnFinishDto.getPreNotice());
            waitingMap.put("preNoticeStaySeconds", teExamUnFinishDto.getPreNoticeStaySeconds());
            waitingMap.put("postNotice", teExamUnFinishDto.getPostNotice());
            finalMap.put("waiting", waitingMap);

            Map<String, Object> activityMap = new HashMap();
            activityMap.put("recordId", teExamUnFinishDto.getRecordId());
            activityMap.put("minDurationSeconds", teExamUnFinishDto.getMinDurationSeconds());
            activityMap.put("cameraPhotoUpload", teExamUnFinishDto.getCameraPhotoUpload());
            activityMap.put("courseCode", teExamUnFinishDto.getCourseCode());
            activityMap.put("examStudentId", teExamUnFinishDto.getExamStudentId());
            activityMap.put("inProcessLivenessVerify", teExamUnFinishDto.getInProcessLivenessVerify());
            activityMap.put("inProcessFaceStrangerIgnore", teExamUnFinishDto.getInProcessFaceStrangerIgnore());
            activityMap.put("inProcessFaceVerify", teExamUnFinishDto.getInProcessFaceVerify());
            activityMap.put("courseName", teExamUnFinishDto.getCourseName());
            activityMap.put("inProcessLivenessFixedRange", teExamUnFinishDto.getInProcessLivenessFixedRange());
            activityMap.put("prepareSeconds", teExamUnFinishDto.getPrepareSeconds());
            activityMap.put("fouceFinish", teExamUnFinishDto.getFouceFinish());
            activityMap.put("entryAuthenticationPolicy", teExamUnFinishDto.getEntryAuthenticationPolicy());
            activityMap.put("maxDurationSeconds", teExamUnFinishDto.getMaxDurationSeconds());
            activityMap.put("examCount", teExamUnFinishDto.getExamCount());
            activityMap.put("inProcessLivenessJudgePolicy", teExamUnFinishDto.getInProcessLivenessJudgePolicy());
            activityMap.put("id", teExamUnFinishDto.getExamActivityId());

            finalMap.put("activity", activityMap);
            return finalMap;
        } else {
            return null;
        }
    }

    @Transactional
    @Override
    public Long saveByPrepare(Long examId, Long examActivityId, Long examStudentId, Long paperId,
                              Integer serialNumber) {
        TOeExamRecord er = new TOeExamRecord();
        er.setId(Constants.idGen.next());
        er.setExamId(examId);
        er.setExamActivityId(examActivityId);
        er.setExamStudentId(examStudentId);
        er.setPaperId(paperId);
        er.setSerialNumber(serialNumber);
        er.setFirstPrepareTime(new Date());
        er.setStatus(ExamRecordStatusEnum.FIRST_PREPARE);
        er.setObjectiveScore(0.0);
        Map<String, Object> map=SimpleBeanUtil.objectToMap(er);
        redisUtil.setForHash(RedisKeyHelper.examRecordCacheKey(er.getId()), map);
        dataInitMq(map);
        return er.getId();
    }

    /**
     * 计算客观分
     */
    @Override
    public void calculateObjectiveScore(Map<String, Object> param) {
        Long recordId = (Long) param.get("recordId");
        Integer mainNumber = (Integer) param.get("mainNumber");
        Integer subNumber = (Integer) param.get("subNumber");
        Integer subIndex = (Integer) param.get("subIndex");
        Long paperId = ExamRecordCacheUtil.getPaperId(recordId);
        Map<String, ObjectiveAnswerCacheBean> map = examPaperService.getObjectiveAnswerCacheBean(paperId);
        if (map == null || map.size() == 0) {
            log.error("no ObjectiveAnswerCacheBean for calculateObjectiveScore");
            return;
        }
        String key = RedisKeyHelper.examAnswerHashKey(mainNumber, subNumber, subIndex);
        ObjectiveAnswerCacheBean cb = map.get(key);
        if (cb == null) {
            return;
        }
        ExamStudentAnswerCacheBean answer = (ExamStudentAnswerCacheBean) redisUtil.get(
                RedisKeyHelper.examAnswerKey(recordId),
                RedisKeyHelper.examAnswerHashKey(mainNumber, subNumber, subIndex));
        if (answer == null) {
            log.error("no ExamStudentAnswerCacheBean for calculateObjectiveScore");
            return;
        }

        if (cb.getStructType().intValue() == 1) {
            if (checkSingleChoice(answer.getAnswer(), cb.getChoiceAnswer())) {
                answer.setScore(cb.getScore());
            } else {
                answer.setScore(0.0);
            }
        }
        if (cb.getStructType().intValue() == 2) {
            ExamCacheBean ec = examService.getExamCacheBean(ExamRecordCacheUtil.getExamId(recordId));
            int r = checkMultipleChoice(answer.getAnswer(), cb.getChoiceAnswer());
            if (ec.getObjectiveScorePolicy().equals(ObjectiveScorePolicyEnum.EQUAL)) {// 全对给分
                if (r == 1) {
                    answer.setScore(cb.getScore());
                } else {
                    answer.setScore(0.0);
                }
            }
            if (ec.getObjectiveScorePolicy().equals(ObjectiveScorePolicyEnum.PARTIAL)) {// 漏选半分
                if (r == 1) {
                    answer.setScore(cb.getScore());
                } else if (r == 0) {
                    BigDecimal b = new BigDecimal(cb.getScore()).divide(new BigDecimal("2"), 1, BigDecimal.ROUND_UP);
                    answer.setScore(b.doubleValue());
                } else {
                    answer.setScore(0.0);
                }
            }
        }
        if (cb.getStructType().intValue() == 3) {
            if (Boolean.parseBoolean(answer.getAnswer()) == cb.getBoolAnswer().booleanValue()) {
                answer.setScore(cb.getScore());
            } else {
                answer.setScore(0.0);
            }
        }

        answer.setObjective(true);
        // 更新分数
        redisUtil.set(RedisKeyHelper.examAnswerKey(recordId),
                RedisKeyHelper.examAnswerHashKey(mainNumber, subNumber, subIndex), answer);

        // 计算客观分总分
        calculateTotalObjectiveScore(recordId, map);
    }

    @SuppressWarnings("unchecked")
    private void calculateTotalObjectiveScore(Long recordId, Map<String, ObjectiveAnswerCacheBean> map) {
        String lockKey = SystemConstant.REDIS_LOCK_TOTAL_OBJECTIVE_SCORE_PREFIX + recordId;
        try {
            Boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_CACHE_TIME_OUT);
            if (lock) {
                Map<String, ExamStudentAnswerCacheBean> as = redisUtil.getHashEntries(RedisKeyHelper.examAnswerKey(recordId));
                if (as != null && as.size() > 0) {
                    Double total = 0.0;
                    for (ExamStudentAnswerCacheBean sa : as.values()) {
                        if (sa.getObjective() != null && sa.getObjective()) {//是客观题
                            if (sa.getScore() != null) {//有分值
                                total = total + sa.getScore();
                            } else {
                                total = null;
                                break;
                            }
                        }
                    }
                    if (total != null) {
                        ExamRecordCacheUtil.setObjectiveScore(recordId, total);
                    }
                }
            }
        } finally {
            redisUtil.releaseLock(lockKey);
        }
    }

    private boolean checkSingleChoice(String answerArray, JSONArray ar) {
    	JSONArray an=JSONArray.parseArray(answerArray);
        int a = an.getIntValue(0);
        return checkSingleChoice(a, ar);
    }
    private boolean checkSingleChoice(int answer, JSONArray ar) {
        for (int i = 0; i < ar.size(); i++) {
            if (answer == ar.getIntValue(i)) {
                return true;
            }
        }
        return false;
    }

    /**
     * @param answer
     * @param ar
     * @return -1 有错选项，0-无措选项 未答全 1-全对
     */
    private int checkMultipleChoice(String answer, JSONArray ar) {
        int yes = 0;
        int no = 0;
        JSONArray an=JSONArray.parseArray(answer);
        for(int i=0;i<an.size();i++) {
            if (checkSingleChoice(an.getIntValue(i), ar)) {
                yes++;
                continue;
            } else {
                no++;
            }
        }
        if (yes == ar.size()) {
            return 1;
        }
        if (yes != 0 && no == 0) {
            return 0;
        }
        return -1;
    }

	/**
	 *考试记录数据持久化
	 */
    @SuppressWarnings("unchecked")
    @Transactional
	@Override
	public void saveDataByCache(Long recordId) {
        String tempDir = SystemConstant.TEMP_FILES_DIR;
        String dir = tempDir + "/" + uuid() + "/";
        File dfile = new File(dir);
        try {
            dfile.mkdirs();
            String structFilePath = sdf.format(new Date()) + "/" + uuid() + ".json";
	        
        	//保存考试记录
        	Map<String,Object> record=redisUtil.getHashEntries(RedisKeyHelper.examRecordCacheKey(recordId));
        	TOeExamRecord er=SimpleBeanUtil.mapToObject(record, TOeExamRecord.class);
        	er.setStatus(ExamRecordStatusEnum.PERSISTED);
        	er.setPaperStructPath(structFilePath);
        	er.setPaperStructUpload(1);
        	saveOrUpdate(er);
        	//保存作答
        	Map<String,ExamStudentAnswerCacheBean> answerMap=redisUtil.getHashEntries(RedisKeyHelper.examAnswerKey(recordId));
        	for(ExamStudentAnswerCacheBean answerCache:answerMap.values()) {
        		TOeExamAnswer answer=new TOeExamAnswer();
        		BeanUtils.copyProperties(answerCache, answer);
        		answer.setId(Constants.idGen.next());
        		answer.setExamRecordId(recordId);
        		examAnswerService.saveOrUpdate(answer);
        	}
        	//更新考生信息
        	ExamStudentCacheBean examStudentCache = examStudentService.getExamStudentCacheBean(er.getExamStudentId());
        	TEExamStudent examStudent=new TEExamStudent();
    		BeanUtils.copyProperties(examStudentCache, examStudent);
    		examStudentService.saveOrUpdate(examStudent);
        	//上传个人试卷结构
            ExamStudentPaperStructCacheBean struct=(ExamStudentPaperStructCacheBean)redisUtil.get(RedisKeyHelper.studentPaperStructKey(recordId));
            File structFile = new File(dir+"struct.json");
            FileUtil.saveAsFile(structFile.getAbsolutePath(), struct.getContent());
            SystemConfig systemConfig = SpringContextHolder.getBean(SystemConfig.class);
            OssUtil.ossUpload(systemConfig.getOssEnv(3), structFilePath, structFile);
        } finally {
            FileUtil.deleteFolder(dir);
        }
	}
	private String uuid() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }
	
	
	@Override
	public void saveFaceVerify(ExamTypeEnum type,Long recordId, Long entryAuthenticationId, VerifyExceptionEnum entryAuthenticationResult) {
		if(ExamTypeEnum.FIRST_START.equals(type)) {
			ExamRecordCacheUtil.setEntryAuthenticationId(recordId, entryAuthenticationId);
			ExamRecordCacheUtil.setEntryAuthenticationResult(recordId, entryAuthenticationResult);
		}else if(ExamTypeEnum.IN_PROCESS.equals(type)) {
			
		}
	}
	
	@Override
	public void saveLivenessVerify(LivenessTypeEnum type,Long recordId, Long entryAuthenticationId, VerifyExceptionEnum entryAuthenticationResult) {
		if(LivenessTypeEnum.FIRST_START.equals(type)) {
			ExamRecordCacheUtil.setEntryAuthenticationId(recordId, entryAuthenticationId);
			ExamRecordCacheUtil.setEntryAuthenticationResult(recordId, entryAuthenticationResult);
		}else if(LivenessTypeEnum.IN_PROCESS.equals(type)) {
			Integer count=ExamRecordCacheUtil.getInProcessLivenessVerifyCount(recordId);
			ExamRecordCacheUtil.setInProcessLivenessVerifyCount(recordId, (count==null?0:count+1));
		}
	}
	
	@Override
	public void dataUpdateMq(Long recordId,String colName,Object colValue) {
		dataUpdateMq(recordId, colName, colValue, 0);
	}
	@Override
	public void dataUpdateMq(Long recordId,String colName,Object colValue,Integer isDate) {
		Map<String, Object> transMap = new HashMap<String, Object>();
		transMap.put("recordId", recordId);
        transMap.put("colName", colName);
        transMap.put("colValue", colValue);
        transMap.put("isDate", isDate);
        //mq发送消息start
        MqDto mqDto = new MqDto(MqTopicEnum.themisTopic.getCode(), MqTagEnum.EXAM_RECORD_UPDATE.name(), transMap, MqTagEnum.EXAM_RECORD_UPDATE, recordId.toString(), colName);
        mqDtoService.assembleSendOneWayMsg(mqDto);
	}

	@Transactional
	@Override
	public void dataUpdate(Long recordId, String colName, Object colValue,Integer isDate) {
		if(isDate!=null&&isDate.intValue()==1) {
			Long l=(Long)colValue;
			Date d=new Date(l);
			tOeExamRecordMapper.dataUpdate(recordId, colName, d);
		}else {
			tOeExamRecordMapper.dataUpdate(recordId, colName, colValue);
		}
	}
	
	@Override
	public void dataInitMq(Map<String, Object> param) {
		Long id=(Long)param.get("id");
        //mq发送消息start
        MqDto mqDto = new MqDto(MqTopicEnum.themisTopic.getCode(), MqTagEnum.EXAM_RECORD_INIT.name(), param, MqTagEnum.EXAM_RECORD_INIT, id.toString(), id.toString());
        mqDtoService.assembleSendOneWayMsg(mqDto);
	}

	@Transactional
	@Override
	public void dataInit(Map<String, Object> param) {
		TOeExamRecord tr=new TOeExamRecord();
		tr.setId((Long)param.get("id"));
		tr.setExamId((Long)param.get("examId"));
		tr.setExamActivityId((Long)param.get("examActivityId"));
		tr.setExamStudentId((Long)param.get("examStudentId"));
		tr.setPaperId((Long)param.get("paperId"));
		tr.setSerialNumber((Integer)param.get("serialNumber"));
		tr.setFirstPrepareTime(new Date((Long)param.get("firstPrepareTime")));
		tr.setStatus(ExamRecordStatusEnum.valueOf((String)param.get("status")));
		tr.setObjectiveScore(0.0);
		saveOrUpdate(tr);
	}
	
	@Override
	public BasePage invigilatePageList(InvigilateListQueryBean param) {
		Page<InvigilateListBean> ipage=new Page<>(param.getPageNumber(), param.getPageSize());
		ipage.addOrder(OrderItem.asc("t.id"));
		tOeExamRecordMapper.invigilatePageQuery(ipage, param);
        BasePage page = new BasePage(ipage.getRecords(), ipage.getCurrent(), ipage.getSize(), ipage.getTotal());
        return page;
	}
}
